unit TestStringConstantCodePagesUnit;

interface

uses
  TestFramework;

// some consts that are internal to the System unit
const
  CP_UTF8 = 65001; // UTF8String CodePage
  CP_UTF16 = 1200; // string CodePage

// some literals that are internal to the System unit
const
  CP_ACP = 0; // default AnsiString CodePage
  CP_RawByteString = $ffff; // RawByteString CodePage


// for CodePage numbers, see http://en.wikipedia.org/wiki/Code_page
// and http://en.wikipedia.org/wiki/Windows_code_pages
const
  CP_VietnameseString = 1258;

type
  VietnameseString = type AnsiString(CP_VietnameseString);

// literal and expression string consts: the LF is always literal, the TwoLF always expression
const
  LF_string = string(#10);
  TwoLF_string = string(LF_string + LF_string);

  LF_AnsiString = AnsiString(#10);
  TwoLF_AnsiString = AnsiString(LF_AnsiString + LF_AnsiString);

  LF_UTF8String = UTF8String(#10);
  TwoLF_UTF8String = UTF8String(LF_UTF8String + LF_UTF8String);

  LF_RawByteString = RawByteString(#10);
  TwoLF_RawByteString = RawByteString(LF_RawByteString + LF_RawByteString);

  LF_VietnameseString = VietnameseString(#10);
  TwoLF_VietnameseString = VietnameseString(LF_VietnameseString + LF_VietnameseString);

type
  TTestStringConstantCodePages = class(TTestCase)
  public
    procedure SetUp; override;
    procedure TearDown; override;
  published
    procedure TestLiteralStringConst;
    procedure TestConcatenatedStringConst;
    procedure TestLiteralAnsiStringConst;
    procedure TestConcatenatedAnsiStringConst;
    procedure TestLiteralUTF8StringConst;
    procedure TestConcatenatedUTF8StringConst;
    procedure TestLiteralRawByteStringConst;
    procedure TestConcatenatedRawByteStringConst;
    procedure TestLiteralVietnameseStringConst;
    procedure TestConcatenatedVietnameseStringConst;
  end;


implementation

procedure TTestStringConstantCodePages.SetUp;
begin
  inherited SetUp();

  // no need for SetUp: everything is in the consts
end;

procedure TTestStringConstantCodePages.TearDown;
begin
  // no need for TearDown: everything is in the consts

  inherited TearDown();
end;

procedure TTestStringConstantCodePages.TestLiteralStringConst;
var
  CP_LF_string: Integer;
begin
  CP_LF_string := StringCodePage(LF_string);

  Self.CheckEquals(CP_UTF16, CP_LF_string,
    'compiler error: CodePage of string const does not match CP_UTF16');
end;

procedure TTestStringConstantCodePages.TestConcatenatedStringConst;
var
  CP_LF_string: Integer;
  CP_TwoLF_string: Integer;
begin
  CP_LF_string := StringCodePage(LF_string);
  CP_TwoLF_string := StringCodePage(TwoLF_string);

  Self.CheckEquals(CP_LF_string, CP_TwoLF_string,
    'compiler error: CodePage of concatenated string const does not match CodePage of string const');
end;

procedure TTestStringConstantCodePages.TestLiteralAnsiStringConst;
var
  CP_LF_AnsiString: Integer;
begin
  CP_LF_AnsiString := StringCodePage(LF_AnsiString);

  Self.CheckEquals(DefaultSystemCodePage, CP_LF_AnsiString,
    'compiler error: CodePage of AnsiString const does not match DefaultSystemCodePage');
end;

procedure TTestStringConstantCodePages.TestConcatenatedAnsiStringConst;
var
  CP_LF_AnsiString: Integer;
  CP_TwoLF_AnsiString: Integer;
begin
  CP_LF_AnsiString := StringCodePage(LF_AnsiString);
  CP_TwoLF_AnsiString := StringCodePage(TwoLF_AnsiString);

  Self.CheckEquals(CP_LF_AnsiString, CP_TwoLF_AnsiString,
    'compiler error: CodePage of concatenated AnsiString const does not match CodePage of AnsiString const');
end;

procedure TTestStringConstantCodePages.TestLiteralUTF8StringConst;
var
  CP_LF_UTF8String: Integer;
begin
  CP_LF_UTF8String := StringCodePage(LF_UTF8String);

  Self.CheckEquals(CP_UTF8, CP_LF_UTF8String,
    'compiler error: CodePage of UTF8String const does not match CP_UTF8');
end;

procedure TTestStringConstantCodePages.TestConcatenatedUTF8StringConst;
var
  CP_LF_UTF8String: Integer;
  CP_TwoLF_UTF8String: Integer;
begin
  CP_LF_UTF8String := StringCodePage(LF_UTF8String);
  CP_TwoLF_UTF8String := StringCodePage(TwoLF_UTF8String);

  Self.CheckEquals(CP_LF_UTF8String, CP_TwoLF_UTF8String,
    'compiler error: CodePage of concatenated UTF8String const does not match CodePage of UTF8String const');
end;

procedure TTestStringConstantCodePages.TestLiteralRawByteStringConst;
var
  CP_LF_RawByteString: Integer;
begin
  CP_LF_RawByteString := StringCodePage(LF_RawByteString);

  Self.CheckEquals(CP_RawByteString, CP_LF_RawByteString,
    'compiler error: CodePage of RawByteString const does not match CP_RawByteString');
end;

procedure TTestStringConstantCodePages.TestConcatenatedRawByteStringConst;
var
  CP_LF_RawByteString: Integer;
  CP_TwoLF_RawByteString: Integer;
begin
  CP_LF_RawByteString := StringCodePage(LF_RawByteString);
  CP_TwoLF_RawByteString := StringCodePage(TwoLF_RawByteString);

  Self.CheckEquals(CP_LF_RawByteString, CP_TwoLF_RawByteString,
    'compiler error: CodePage of concatenated RawByteString const does not match CodePage of RawByteString const');
end;

procedure TTestStringConstantCodePages.TestLiteralVietnameseStringConst;
var
  CP_LF_VietnameseString: Integer;
begin
  CP_LF_VietnameseString := StringCodePage(LF_VietnameseString);

  Self.CheckEquals(CP_VietnameseString, CP_LF_VietnameseString,
    'compiler error: CodePage of VietnameseString const does not match CP_VietnameseString');
end;

procedure TTestStringConstantCodePages.TestConcatenatedVietnameseStringConst;
var
  CP_LF_VietnameseString: Integer;
  CP_TwoLF_VietnameseString: Integer;
begin
  CP_LF_VietnameseString := StringCodePage(LF_VietnameseString);
  CP_TwoLF_VietnameseString := StringCodePage(TwoLF_VietnameseString);

  Self.CheckEquals(CP_LF_VietnameseString, CP_TwoLF_VietnameseString,
    'compiler error: CodePage of concatenated VietnameseString const does not match CodePage of VietnameseString const');
end;

initialization
  // Register any test cases with the test runner
  RegisterTest(TTestStringConstantCodePages.Suite);
end.
