﻿namespace bo.Xml;

interface

uses
  System.Collections.Generic,
  System.IO,
  System.Text,
  System.Xml,
  System.Xml.Schema;

type
  XmlReadEventArgs = public class(EventArgs)
  private 
    FReader: XmlReader;
  public
    constructor(reader: XmlReader);
    property Reader: XmlReader read FReader;
  end;
  
  XmlValidationEventArgs = public class(EventArgs)
  private 
    FE: ValidationEventArgs;
    FReader: XmlReader;
  public
    constructor (reader: XmlReader; e: ValidationEventArgs);
    property  E: ValidationEventArgs read FE;
    property Reader: XmlReader read FReader;
  end;

  XmlValidator = public class
  public
    event XmlReadEventHandler: XmlReadEventHandler;
    event XmlValidationEventHandler: XmlValidationEventHandler;
    method ValidateXml(xmlFileName: string; xsdFileName: string); virtual;
    method ValidateXml(xmlFileName: string; xsdFileNames: array of string); virtual;
    method ValidationResult: string; virtual;
    /// <summary>
    /// Hook self up to XmlValidationEventArgs if you want to fill ValidationResult
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    method ValidationResultEventHandler(sender: object; e: XmlValidationEventArgs);
  protected 
    method OnReadEventHandler(); virtual;
    method reader_ValidationEventHandler(sender: object; e: ValidationEventArgs); virtual;
    method GetCurrentNodeAsString(reader: XmlReader): string; virtual;
  private 
    FReader: XmlReader;
    FCurrentLine: String;
    FValidationResult: StringBuilder;
  end;

  XmlReadEventHandler = public delegate (sender: Object; e: XmlReadEventArgs);
  XmlValidationEventHandler = public delegate (sender: Object; e: XmlValidationEventArgs);

implementation

constructor XmlReadEventArgs(reader: XmlReader);
begin
  inherited constructor();
  self.FReader := reader;
end;

constructor XmlValidationEventArgs(reader: XmlReader; e: ValidationEventArgs);
begin
  inherited constructor();
  self.FE := e;
  self.FReader := reader;
end;

method XmlValidator.ValidateXml(xmlFileName: string; xsdFileName: string);
begin
  ValidateXml(xmlFileName, [xsdFileName]);
end;

method XmlValidator.ValidateXml(xmlFileName: string; xsdFileNames: array of string);
begin
  var xsdFileStreamsToDispose := new List<FileStream>();

  try
    var settings := new XmlReaderSettings();
    settings.ValidationType := ValidationType.Schema;

    for each xsdFileName in xsdFileNames do
    begin
      // resolve the relative SchemaLocation of the included Xsd's using the
      // base path of the xsdFileName
      // see http://www.hanselman.com/blog/ValidatingXMLSchemasThatHaveBeenXsimportedAndHaveSchemaLocation.aspx
      // Refactoring idee: in de toekomst oplossen met een resolver (die recursief werkt)
      // see http://www.hanselman.com/blog/ValidatingThatXmlSchemasAndTheirImportsAreValidAndAllGoodWithAnXmlResolver.aspx

      var xsdFileStream := File.OpenRead(xsdFileName);
      xsdFileStreamsToDispose.Add(xsdFileStream);
      var schema := XmlSchema.Read(xsdFileStream, nil);

      var baseDirectory := Path.GetDirectoryName(xsdFileName);
      for each include: XmlSchemaInclude in schema.Includes do
      begin
        // if include.SchemaLocation is an absolute path, Path.Combine returns that,
        // otherwise it appends the relative path to the baseDirectory
        include.SchemaLocation := Path.Combine(baseDirectory, include.SchemaLocation);
      end;

      settings.Schemas.Add(schema);
    end;

    settings.ValidationEventHandler +=
        new ValidationEventHandler(reader_ValidationEventHandler);

    using xmlFile := File.OpenRead(xmlFileName) do
    begin
        using reader := XmlReader.Create(xmlFile, settings) do
        begin
          FReader  := reader;
          while (
            FReader.Read()
          )
          do
          begin
            FCurrentLine := GetCurrentNodeAsString(FReader);
            OnReadEventHandler();
          end
        end;
        FReader := nil;
    end;
  finally
    for each fileStream in xsdFileStreamsToDispose do
      fileStream.Dispose();
  end
end;

method XmlValidator.OnReadEventHandler();
begin
  if assigned(XmlReadEventHandler) then
    XmlReadEventHandler(self, new XmlReadEventArgs(FReader));
end;

method XmlValidator.reader_ValidationEventHandler(sender: object; e: ValidationEventArgs);
begin
  if assigned(XmlValidationEventHandler) then
    XmlValidationEventHandler(self, new XmlValidationEventArgs(FReader, e));
end;

method XmlValidator.ValidationResult: string;
begin
  if Assigned(FValidationResult) then
    Result := FValidationResult.ToString()
  else
    Result := nil;
end;

method XmlValidator.ValidationResultEventHandler(sender: object; e: XmlValidationEventArgs);
begin
  if not assigned(FValidationResult) then
    FValidationResult := new StringBuilder();

  FValidationResult.AppendFormat("{0}: {1}", e.E.Severity.ToString(), e.E.Message);
  FValidationResult.AppendLine();

  if not string.IsNullOrEmpty(FCurrentLine) then
    FValidationResult.AppendLine(FCurrentLine);

  var Line := GetCurrentNodeAsString(e.Reader);
  if not string.IsNullOrEmpty(Line) then
    FValidationResult.AppendLine(Line);
end;

method XmlValidator.GetCurrentNodeAsString(reader: XmlReader): string;
begin
  Result := ''; 
  if assigned(FReader) then
  begin
    case FReader.NodeType of
      XmlNodeType.Element:
        Result := String.Format("<{0}>", reader.Name);
      XmlNodeType.Text:
        Result := reader.Value;
      XmlNodeType.CDATA:
        Result := String.Format("<![CDATA[{0}]]>", reader.Value);
      XmlNodeType.ProcessingInstruction:
        Result := String.Format("<?{0} {1}?>", reader.Name, reader.Value);
      XmlNodeType.Comment:
        Result := String.Format("<!--{0}-->", reader.Value);
      XmlNodeType.XmlDeclaration:
        Result := String.Format("<?{0} {1}?>", reader.Name, reader.Value);
      XmlNodeType.Document:
        Result := '';
      XmlNodeType.DocumentType:
        Result := String.Format("<!DOCTYPE {0} [{1}]", reader.Name, reader.Value);
      XmlNodeType.EntityReference:
        Result := reader.Name;
      XmlNodeType.EndElement:
        Result := String.Format("</{0}>", reader.Name);
    end;
  end;
end;

end.

